/*!
 * @vueblocks/vue-use-core v0.2.0
 * (c) 2021 xiaoluoboding
 * @license MIT
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vueDemi = require('vue-demi');
var debounce = require('lodash.debounce');
var screenfull = require('screenfull');
var throttle = require('lodash.throttle');
var keymaster = require('keymaster');
var resizeObserver = require('@juggle/resize-observer');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var debounce__default = /*#__PURE__*/_interopDefaultLegacy(debounce);
var screenfull__default = /*#__PURE__*/_interopDefaultLegacy(screenfull);
var throttle__default = /*#__PURE__*/_interopDefaultLegacy(throttle);
var keymaster__default = /*#__PURE__*/_interopDefaultLegacy(keymaster);

const useDebouncedRef = (value, delay = 200) => {
    return vueDemi.customRef((track, trigger) => {
        return {
            get() {
                track();
                return value;
            },
            set: debounce__default['default']((newValue) => {
                value = newValue;
                trigger();
            }, delay)
        };
    });
};

/**
 * Call onMounted() if it's inside a component lifecycle, if not, run just call the function
 * @param fn
 * @param sync if set to false, it will run in the nextTick() of Vue
 */
function tryOnMounted(fn, sync = true) {
    if (vueDemi.getCurrentInstance()) {
        vueDemi.onMounted(fn);
    }
    else if (sync) {
        fn();
    }
    else {
        vueDemi.nextTick(fn);
    }
}
/**
 * Call onUnmounted() if it's inside a component lifecycle, if not, do nothing
 * @param fn
 */
function tryOnUnmounted(fn) {
    if (vueDemi.getCurrentInstance())
        vueDemi.onUnmounted(fn);
}

/**
 * Reactive Fullscreen API build on top of screenfull.js
 * @param {HTMLElement} target
 * @return {boolean} isFullscreen
 * @return {function} toggleFullscreen
 */
const useFullscreen = (target) => {
    const targetRef = vueDemi.ref(target || document.querySelector('html'));
    const isFullscreen = vueDemi.ref(false);
    const toggleFullscreen = () => {
        screenfull__default['default'].isEnabled && screenfull__default['default'].toggle(targetRef.value);
    };
    tryOnMounted(() => {
        if (screenfull__default['default'].isEnabled) {
            screenfull__default['default'].on('change', () => {
                isFullscreen.value = screenfull__default['default'].isFullscreen;
            });
        }
    });
    return {
        isFullscreen,
        toggleFullscreen
    };
};

function useReactiveState(getter) {
    const state = vueDemi.reactive(getter());
    const update = () => {
        Object.assign(state, getter());
    };
    return [state, update];
}

/**
 * Reactive Resize API, When window resize, do someting.
 * @param {function} fn
 * @param {boolean} useCapture
 * @param {boolean} isThrottle: true: throttle | false: debounce
 * @return {function} dispatchResize
 */
const useResizeEvent = (fn, useCapture = false, isThrottle = false) => {
    const dispatchResize = () => {
        const e = document.createEvent('Event');
        e.initEvent('resize', true, true);
        window.dispatchEvent(e);
    };
    const waitFn = isThrottle ? throttle__default['default'](fn, 500) : debounce__default['default'](fn, 500);
    tryOnMounted(() => {
        window.addEventListener('resize', waitFn, useCapture);
    });
    tryOnUnmounted(() => {
        window.removeEventListener('resize', waitFn, useCapture);
    });
    return {
        dispatchResize
    };
};

const bindKeyHandler = (fn) => {
    return (...args) => {
        fn(...args);
        return false;
    };
};
/**
 * Given shortcut a new bind function
 */
const shortcut = {
    bind: (seed, func) => keymaster__default['default'](seed, bindKeyHandler(func)),
    ...keymaster__default['default']
};
/**
 * Bind shortcut onMounted & unbind shortcut automatically onUnmounted in setup hooks
 * or
 * Unbind shortcut before bind shortcut without setup hooks
 * @param {Object} keymap
 * @param {Object} vm
 * @return {function} toggle
 */
const useShortcut = (keymap, vm) => {
    let sync = true;
    if (!vm) {
        vm = vueDemi.getCurrentInstance();
    }
    else {
        sync = false;
        unbindKeys();
    }
    function bindKeys() {
        Object.keys(keymap).forEach(key => {
            shortcut.bind(key, keymap[key]);
        });
    }
    function unbindKeys() {
        Object.keys(keymap).forEach(key => {
            shortcut.unbind(key);
        });
    }
    tryOnMounted(() => bindKeys(), sync);
    tryOnUnmounted(unbindKeys);
    return shortcut;
};

/**
 * It immediately detects when an element resizes and provides accurate sizing information back to the handler.
 * @param target
 * @param callback
 * @param options
 */
const useResizeObserver = (target, callback, options) => {
    let ro;
    const targetRef = vueDemi.ref(target);
    const isSupported = window && 'ResizeObserver' in window;
    const stopWatch = vueDemi.watch(targetRef, (newVal) => {
        if (isSupported && window && newVal) {
            ro = new resizeObserver.ResizeObserver(callback);
            ro.observe(newVal, options);
        }
    }, { immediate: true });
    const stop = () => {
        ro?.disconnect();
        stopWatch();
    };
    return {
        stop
    };
};

exports.tryOnMounted = tryOnMounted;
exports.tryOnUnmounted = tryOnUnmounted;
exports.useDebouncedRef = useDebouncedRef;
exports.useFullscreen = useFullscreen;
exports.useReactiveState = useReactiveState;
exports.useResizeEvent = useResizeEvent;
exports.useResizeObserver = useResizeObserver;
exports.useShortcut = useShortcut;
