import { Ref } from 'vue-demi';
import { ResizeObserver } from '@juggle/resize-observer';

declare const useDebouncedRef: (value: any, delay?: number) => Ref;

declare type Fn = () => void;

interface IFullScreen {
    isFullscreen: Ref<boolean>;
    toggleFullscreen: Fn;
}
/**
 * Reactive Fullscreen API build on top of screenfull.js
 * @param {HTMLElement} target
 * @return {boolean} isFullscreen
 * @return {function} toggleFullscreen
 */
declare const useFullscreen: (target: HTMLElement) => IFullScreen;

/**
 * Call onMounted() if it's inside a component lifecycle, if not, run just call the function
 * @param fn
 * @param sync if set to false, it will run in the nextTick() of Vue
 */
declare function tryOnMounted(fn: Fn, sync?: boolean): void;
/**
 * Call onUnmounted() if it's inside a component lifecycle, if not, do nothing
 * @param fn
 */
declare function tryOnUnmounted(fn: Fn): void;

declare function useReactiveState(getter: () => Required<object>): [Required<object>, VoidFunction];

/**
 * Reactive Resize API, When window resize, do someting.
 * @param {function} fn
 * @param {boolean} useCapture
 * @param {boolean} isThrottle: true: throttle | false: debounce
 * @return {function} dispatchResize
 */
declare const useResizeEvent: (fn: Fn, useCapture?: boolean, isThrottle?: boolean) => {
    dispatchResize: () => void;
};

/**
 * A map of keybinding strings to event handlers.
 */
interface KeyBindingMap {
    [keybinding: string]: (event: KeyboardEvent) => void;
}
interface KeymasterEvent {
    key: string;
    method: KeyHandler;
    mods: number[];
    scope: string;
    shortcut: string;
}
interface KeyHandler {
    (keyboardEvent: KeyboardEvent, keymasterEvent: KeymasterEvent): void;
}
/**
 * Bind shortcut onMounted & unbind shortcut automatically onUnmounted in setup hooks
 * or
 * Unbind shortcut before bind shortcut without setup hooks
 * @param {Object} keymap
 * @param {Object} vm
 * @return {function} toggle
 */
declare const useShortcut: (keymap: KeyBindingMap, vm: any) => {
    shift: boolean;
    alt: boolean;
    option: boolean;
    ctrl: boolean;
    control: boolean;
    command: boolean;
    setScope(scopeName: string): void;
    getScope(): string;
    deleteScope(scopeName: string): void;
    noConflict(): void;
    unbind(key: string): void;
    unbind(key: string, scopeName: string): void;
    isPressed(keyCode: number): boolean;
    getPressedKeyCodes(): number[];
    filter(event: FilterEvent): void;
    bind: (seed: string, func: Function) => void;
};

/**
 * Size of a specific box.
 *
 * https://drafts.csswg.org/resize-observer-1/#resizeobserversize
 */
interface ResizeObserverSize {
    readonly inlineSize: number;
    readonly blockSize: number;
}
/**
 * Used to observe different box sizes.
 *
 * https://drafts.csswg.org/resize-observer-1/#enumdef-resizeobserverboxoptions
 */
declare enum ResizeObserverBoxOptions {
    BORDER_BOX = "border-box",
    CONTENT_BOX = "content-box",
    DEVICE_PIXEL_CONTENT_BOX = "device-pixel-content-box"
}
/**
 * Options to be given to the resize observer,
 * when observing a new element.
 *
 * https://drafts.csswg.org/resize-observer-1/#dictdef-resizeobserveroptions
 */
interface ResizeObserverOptions {
    box?: 'content-box' | 'border-box' | 'device-pixel-content-box' | ResizeObserverBoxOptions;
}
/**
 * https://drafts.csswg.org/resize-observer-1/#resize-observer-entry-interface
 */
interface ResizeObserverEntry {
    readonly target: Element;
    readonly contentRect: DOMRectReadOnly;
    readonly borderBoxSize?: ReadonlyArray<ResizeObserverSize>;
    readonly contentBoxSize?: ReadonlyArray<ResizeObserverSize>;
    readonly devicePixelContentBoxSize?: ReadonlyArray<ResizeObserverSize>;
}
/**
 * https://drafts.csswg.org/resize-observer-1/#resize-observer-callback
 */
declare type ResizeObserverCallback = (entries: ResizeObserverEntry[], observer: ResizeObserver) => void;

/**
 * It immediately detects when an element resizes and provides accurate sizing information back to the handler.
 * @param target
 * @param callback
 * @param options
 */
declare const useResizeObserver: (target: Element, callback: ResizeObserverCallback, options: ResizeObserverOptions) => {
    stop: () => void;
};

export { IFullScreen, KeyBindingMap, KeyHandler, KeymasterEvent, tryOnMounted, tryOnUnmounted, useDebouncedRef, useFullscreen, useReactiveState, useResizeEvent, useResizeObserver, useShortcut };
